@@ -1,9 +1,10 @@
 ############################################################
 # Copyright (C) 2008 Masahiko SAWAI All Rights Reserved. 
 ############################################################
-
-# button state viewer
-# A + B : exit program.
+MANUAL = <<EOD
+button state viewer
+A + B : exit program.
+EOD
 
 require 'wiiremote'
 
@@ -19,6 +20,8 @@ WRMT::wiiremotes.each(){ |wiiremote|
 	wiiremote.open()
 }
 
+print MANUAL
+
 done = false
 until done do
 	while (updated_index = WRMT::poll()) >= 0 do
