@@ -1,10 +1,12 @@
 ############################################################
 # Copyright (C) 2008 Masahiko SAWAI All Rights Reserved. 
 ############################################################
-
-# 1 : enable force feedback
-# 2 : disable force feedback
-# A + B : quit
+MANUAL = <<EOD
+force feedback test
+1 : enable force feedback
+2 : disable force feedback
+A + B : quit
+EOD
 
 require 'wiiremote'
 
@@ -19,6 +21,7 @@ end
 WRMT::wiiremotes.each(){ |wiiremote|
 	wiiremote.open()
 }
+print MANUAL
 
 done = false
 until done do
@@ -27,10 +30,12 @@ until done do
 		if wiiremote.check_button_any(WRMT::MASK_BUTTON_ONE|WRMT::MASK_BUTTON_TWO) then
 			f = wiiremote[WRMT::DATA_FORCE_FEEDBACK]
 			if f == 0 and wiiremote.check_button(WRMT::MASK_BUTTON_ONE) then
-				wiiremote[WRMT::DATA_FORCE_FEEDBACK] = 1;
+				f = 1
 			elsif f == 1 and wiiremote.check_button(WRMT::MASK_BUTTON_TWO) then
-				wiiremote[WRMT::DATA_FORCE_FEEDBACK] = 0;
+				f = 0
 			end
+			wiiremote[WRMT::DATA_FORCE_FEEDBACK] = f;
+			wiiremote.update()
 		end
 		done = true if wiiremote.check_button_all("AB")
 	end
