@@ -1,6 +1,10 @@
 ############################################################
 # Copyright (C) 2008 Masahiko SAWAI All Rights Reserved. 
 ############################################################
+MANUAL = <<EOD
+ir sensor state viewer
+A + B : exit program.
+EOD
 
 require 'wiiremote'
 
@@ -15,14 +19,21 @@ end
 WRMT::wiiremotes.each(){ |wiiremote|
 	wiiremote.open()
 	wiiremote.ir = true;
+	wiiremote.update()
 }
 
+print MANUAL
+
 done = false
 until done do
 	while (updated_index = WRMT::poll()) >= 0 do
 		wiiremote = WRMT::wiiremotes[updated_index]
-		printf("[%d] : p1(%s, %d, %d, %d) p2(%s, %d, %d, %d)\n",
+		printf("[%d] : (%s, %d, %d, %d) p1(%s, %d, %d, %d) p2(%s, %d, %d, %d)\n",
 			updated_index,
+			wiiremote[WRMT::DATA_IR_FOUND] == 1,
+			wiiremote[WRMT::DATA_IR_SIZE],
+			wiiremote[WRMT::DATA_IR_X],
+			wiiremote[WRMT::DATA_IR_Y],
 			wiiremote[WRMT::DATA_IR1_FOUND] == 1,
 			wiiremote[WRMT::DATA_IR1_SIZE],
 			wiiremote[WRMT::DATA_IR1_X],
