@@ -1,10 +1,13 @@
 ############################################################
 # Copyright (C) 2008 Masahiko SAWAI All Rights Reserved. 
 ############################################################
+MANUAL = <<EOD
+LED test
++ : led += 1
+- : led -= 1
+A + B : quit
+EOD
 
-# + : led += 1
-# - : led -= 1
-# A + B : quit
 
 require 'wiiremote'
 
@@ -20,6 +23,8 @@ WRMT::wiiremotes.each(){ |wiiremote|
 	wiiremote.open()
 }
 
+print MANUAL
+
 done = false
 until done do
 	while (updated_index = WRMT::poll()) >= 0 do
@@ -34,6 +39,7 @@ until done do
 				led = 0x0f
 			end
 			wiiremote[WRMT::DATA_LEDS] = led;
+			wiiremote.update()
 		end
 
 		done = true if wiiremote.check_button_all("AB")
