@@ -7,6 +7,44 @@
 #include <wiiremote.h>
 #include <wiiremote_utils.h>
 
+static
+VALUE
+wiiremotes = Qnil;
+
+static
+int
+button_mask_from_value(VALUE button_mask_value)
+{
+	int button_mask = 0;
+
+	switch (TYPE(button_mask_value))
+	{
+	case T_STRING :
+		{
+			int len = RSTRING(button_mask_value)->len;
+			void *ptr = RSTRING(button_mask_value)->ptr;
+			
+			if (memchr(ptr, '2', len) != NULL) button_mask |= WRMT_MASK_BUTTON_TWO;
+			if (memchr(ptr, '1', len) != NULL) button_mask |= WRMT_MASK_BUTTON_ONE;
+			if (memchr(ptr, 'B', len) != NULL) button_mask |= WRMT_MASK_BUTTON_B;
+			if (memchr(ptr, 'A', len) != NULL) button_mask |= WRMT_MASK_BUTTON_A;
+			if (memchr(ptr, '-', len) != NULL) button_mask |= WRMT_MASK_BUTTON_MINUS;
+			if (memchr(ptr, 'H', len) != NULL) button_mask |= WRMT_MASK_BUTTON_HOME;
+			if (memchr(ptr, 'L', len) != NULL) button_mask |= WRMT_MASK_BUTTON_LEFT;
+			if (memchr(ptr, 'R', len) != NULL) button_mask |= WRMT_MASK_BUTTON_RIGHT;
+			if (memchr(ptr, 'D', len) != NULL) button_mask |= WRMT_MASK_BUTTON_DOWN;
+			if (memchr(ptr, 'U', len) != NULL) button_mask |= WRMT_MASK_BUTTON_UP;
+			if (memchr(ptr, '+', len) != NULL) button_mask |= WRMT_MASK_BUTTON_PLUS;
+		}
+		break;
+	default:
+		button_mask = NUM2INT(button_mask_value);
+		break;
+	}
+
+	return button_mask;
+}
+
 /**
 * WRMT module 
 **/
@@ -16,19 +54,21 @@ VALUE
 WRMT_init(VALUE klass)
 {
 	int rc, number, i;
-	VALUE WiiRemote;
-	VALUE wiiremotes;
+	VALUE WRMT, WiiRemote;
 
 	/* init libwiiremote */
 	rc = WRMT_Init();
 	if (rc != 0)
 	{
-		rb_raise(rb_eStandardError, "libwiiremote initialization failed(code %d)", rc);
+		/* FIXME : use WRMT_GetError() */
+		rb_raise(rb_eStandardError, "libwiiremote initialization failed");
 	}
 
-	/* init array (wiiremotes) */
-	WiiRemote = rb_const_get(klass, rb_intern("WiiRemote"));
-	wiiremotes = rb_ivar_get(klass, rb_intern("wiiremotes"));
+	/* get WRMT::WiiRemote class object */
+	WRMT = rb_const_get(rb_cObject, rb_intern("WRMT"));
+	WiiRemote = rb_const_get(WRMT, rb_intern("WiiRemote"));
+
+	/* init wiiremotes array */
 	rb_ary_clear(wiiremotes);
 	number = WRMT_GetNumWiiRemote();
 	for (i = 0;i < number;i++)
@@ -52,9 +92,9 @@ WRMT_quit(VALUE klass)
 
 static
 VALUE
-WRMT_update(VALUE klass)
+WRMT_update_all(VALUE klass)
 {
-	WRMT_Update();
+	WRMT_UpdateAll();
 	return Qnil;
 }
 
@@ -77,9 +117,7 @@ VALUE
 WRMT_wiiremote_at(VALUE klass, VALUE device_index_value)
 {
 	VALUE result;
-	VALUE wiiremotes;
 
-	wiiremotes = rb_ivar_get(klass, rb_intern("wiiremotes"));
 	result = rb_ary_entry(wiiremotes, NUM2LONG(device_index_value));
 	
 	return result;
@@ -89,7 +127,45 @@ static
 VALUE
 WRMT_wiiremotes(VALUE klass)
 {
-	return rb_ivar_get(klass, rb_intern("wiiremotes"));
+	return wiiremotes;
+}
+
+static
+VALUE
+WRMT_check_button_any(VALUE klass,
+	VALUE button_state_value,
+	VALUE button_mask_value)
+{
+	VALUE result = Qfalse;
+	int button_state, button_mask;
+
+	button_state = NUM2INT(button_state_value);
+	button_mask = button_mask_from_value(button_mask_value);
+	if (WRMT_CheckButtonAny(button_state, button_mask))
+	{
+		result = Qtrue;
+	}
+
+	return result;
+}
+
+static
+VALUE
+WRMT_check_button_all(VALUE klass,
+	VALUE button_state_value,
+	VALUE button_mask_value)
+{
+	VALUE result = Qfalse;
+	int button_state, button_mask;
+
+	button_state = NUM2INT(button_state_value);
+	button_mask = button_mask_from_value(button_mask_value);
+	if (WRMT_CheckButtonAll(button_state_value, button_mask))
+	{
+		result = Qtrue;
+	}
+
+	return result;
 }
 
 
@@ -172,6 +248,24 @@ WiiRemote_close(VALUE self)
 	return Qnil;
 }
 
+static
+VALUE
+WiiRemote_update(VALUE self)
+{
+	WRMT_WiiRemote *wiiremote;
+	WRMT_IOReturn rc;
+
+	Data_Get_Struct(self, WRMT_WiiRemote, wiiremote);
+	rc = WRMT_WiiRemote_Update(wiiremote);
+	if (rc != WRMT_IO_SUCCESS)
+	{
+		/* FIXME : use WRMT_GetError() */
+		rb_raise(rb_eStandardError, "WRMT_WiiRemote_Update() failed");
+	}
+
+	return Qnil;
+}
+
 static
 VALUE
 WiiRemote_enabled_p(VALUE self, VALUE function_type_value)
@@ -229,7 +323,6 @@ VALUE
 WiiRemote_set_enabled(VALUE self, VALUE function_type_value, VALUE value_value)
 {
 	WRMT_WiiRemote *wiiremote;
-	WRMT_IOReturn rc;
 	int function_type;
 	int value;
 
@@ -257,11 +350,7 @@ WiiRemote_set_enabled(VALUE self, VALUE function_type_value, VALUE value_value)
 		if (value != 0) value = 1;
 	}
 
-	rc = WRMT_WiiRemote_SetEnabled(wiiremote, function_type, value);
-	if (rc != WRMT_IO_SUCCESS)
-	{
-		rb_raise(rb_eStandardError, "WRMT_WiiRemote_SetEnabled() failed");
-	}
+	WRMT_WiiRemote_SetEnabled(wiiremote, function_type, value);
 
 	return Qnil;
 }
@@ -304,15 +393,14 @@ WiiRemote_set_speaker(VALUE self, VALUE value_value)
 
 static
 VALUE
-WiiRemote_state(VALUE self, VALUE data_type_value)
+WiiRemote_state_impl(VALUE self, int data_type)
 {
 	VALUE result = Qnil;
 	WRMT_WiiRemote *wiiremote;
-	int data_type, data;
+	int  data;
 
 	Data_Get_Struct(self, WRMT_WiiRemote, wiiremote);
 
-	data_type = NUM2INT(data_type_value);
 	if (data_type < 0 && data_type >= WRMT_NUMBER_OF_DATA_TYPE)
 	{
 		rb_raise(rb_eIndexError, "data type %d is out of range (0 - %d)",
@@ -325,12 +413,75 @@ WiiRemote_state(VALUE self, VALUE data_type_value)
 	return result;
 }
 
+static
+VALUE
+WiiRemote_state(VALUE self, VALUE data_type_value)
+{
+	return WiiRemote_state_impl(self, NUM2INT(data_type_value));
+}
+
+static
+VALUE
+WiiRemote_buttons(VALUE self)
+{
+	return WiiRemote_state_impl(self, WRMT_DATA_BUTTONS);
+}
+
+static
+VALUE
+WiiRemote_motion_x(VALUE self)
+{
+	return WiiRemote_state_impl(self, WRMT_DATA_MOTION_X);
+}
+
+static
+VALUE
+WiiRemote_motion_y(VALUE self)
+{
+	return WiiRemote_state_impl(self, WRMT_DATA_MOTION_Y);
+}
+
+static
+VALUE
+WiiRemote_motion_z(VALUE self)
+{
+	return WiiRemote_state_impl(self, WRMT_DATA_MOTION_Z);
+}
+
+static
+VALUE
+WiiRemote_ir_found(VALUE self)
+{
+	VALUE found_value = WiiRemote_state_impl(self, WRMT_DATA_IR_FOUND);
+	return (NUM2INT(found_value) == 0) ? Qfalse : Qtrue;
+}
+
+static
+VALUE
+WiiRemote_ir_size(VALUE self)
+{
+	return WiiRemote_state_impl(self, WRMT_DATA_IR_SIZE);
+}
+
+static
+VALUE
+WiiRemote_ir_x(VALUE self)
+{
+	return WiiRemote_state_impl(self, WRMT_DATA_IR_X);
+}
+
+static
+VALUE
+WiiRemote_ir_y(VALUE self)
+{
+	return WiiRemote_state_impl(self, WRMT_DATA_IR_Y);
+}
+
 static
 VALUE
 WiiRemote_set_state(VALUE self, VALUE data_type_value, VALUE data_value)
 {
 	WRMT_WiiRemote *wiiremote;
-	WRMT_IOReturn rc;
 	int data_type, data;
 
 	Data_Get_Struct(self, WRMT_WiiRemote, wiiremote);
@@ -344,49 +495,72 @@ WiiRemote_set_state(VALUE self, VALUE data_type_value, VALUE data_value)
 
 	data = NUM2INT(data_value);
 
-	rc = WRMT_WiiRemote_SetState(wiiremote, data_type, data);
-	if (rc != WRMT_IO_SUCCESS)
-	{
-		rb_raise(rb_eStandardError, "WRMT_WiiRemote_SetState() failed");
-	}
+	WRMT_WiiRemote_SetState(wiiremote, data_type, data);
 
 	return Qnil;
 }
 
 static
-int
-button_mask_from_value(VALUE button_mask_value)
+VALUE
+WiiRemote_play_sound(VALUE self, VALUE data_value,
+	VALUE offset_value, VALUE length_value)
 {
-	int button_mask = 0;
+	WRMT_WiiRemote *wiiremote;
+	WRMT_IOReturn rc;
+	VALUE data_string;
+	char *data_ptr;
+	int data_length;
+	int offset, length;
 
-	switch (TYPE(button_mask_value))
+	/* check and convert data parameter */
+	if (data_value == Qnil)
+		rb_raise(rb_eArgError, "data parameter should not be nil");
+
+	data_string = data_value;
+	StringValue(data_string);
+	data_ptr = RSTRING(data_string)->ptr;
+	data_length = RSTRING(data_string)->len;
+
+	/* check and convert offset parameter */
+	offset = 0;
+	if (offset_value != Qnil)
 	{
-	case T_STRING :
+		offset = NUM2INT(offset_value);
+		if (offset < 0 || offset >= data_length)
 		{
-			int len = RSTRING(button_mask_value)->len;
-			void *ptr = RSTRING(button_mask_value)->ptr;
-			
-			if (memchr(ptr, '2', len) != NULL) button_mask |= WRMT_MASK_BUTTON_TWO;
-			if (memchr(ptr, '1', len) != NULL) button_mask |= WRMT_MASK_BUTTON_ONE;
-			if (memchr(ptr, 'B', len) != NULL) button_mask |= WRMT_MASK_BUTTON_B;
-			if (memchr(ptr, 'A', len) != NULL) button_mask |= WRMT_MASK_BUTTON_A;
-			if (memchr(ptr, '-', len) != NULL) button_mask |= WRMT_MASK_BUTTON_MINUS;
-			if (memchr(ptr, 'H', len) != NULL) button_mask |= WRMT_MASK_BUTTON_HOME;
-			if (memchr(ptr, 'L', len) != NULL) button_mask |= WRMT_MASK_BUTTON_LEFT;
-			if (memchr(ptr, 'R', len) != NULL) button_mask |= WRMT_MASK_BUTTON_RIGHT;
-			if (memchr(ptr, 'D', len) != NULL) button_mask |= WRMT_MASK_BUTTON_DOWN;
-			if (memchr(ptr, 'U', len) != NULL) button_mask |= WRMT_MASK_BUTTON_UP;
-			if (memchr(ptr, '+', len) != NULL) button_mask |= WRMT_MASK_BUTTON_PLUS;
+			rb_raise(rb_eIndexError,
+				"offset parameter %d is out of range (0 - %d)",
+				offset, data_length-1);
 		}
-		break;
-	default:
-		button_mask = NUM2INT(button_mask_value);
-		break;
 	}
 
-	return button_mask;
+	/* check and convert length parameter */
+	length = data_length - offset;
+	if (length_value != Qnil)
+	{
+		length = NUM2INT(length_value);
+		if (length <= 0)
+			rb_raise(rb_eArgError, "length parameter must be positive number");
+
+		if (offset + length > data_length)
+			length = data_length - offset;
+	}
+
+	fprintf(stderr, "PlaySound(offset : %d, length : %d)\n", offset, length);
+
+	/* call native function */
+	Data_Get_Struct(self, WRMT_WiiRemote, wiiremote);
+	rc = WRMT_WiiRemote_PlaySound(wiiremote, (const unsigned char *)(data_ptr + offset), length);
+	if (rc != 0)
+	{
+		/* FIXME : use WRMT_GetError() */
+		rb_raise(rb_eStandardError, "WiiRemote_PlaySound() failed");
+	}
+
+	return Qnil;
 }
 
+
 static
 VALUE
 WiiRemote_check_button_any(VALUE self, VALUE button_mask_value)
@@ -429,12 +603,16 @@ void Init_wiiremote()
 {
 	VALUE WRMT, WiiRemote;
 
+	/**
+	* init C global variable
+	**/
+	wiiremotes = rb_ary_new();
+	rb_global_variable(&wiiremotes);
+
 	/**
 	* WRMT
 	**/
 	WRMT = rb_define_module("WRMT");
-	rb_ivar_set(WRMT, rb_intern("wiiremotes"), rb_ary_new());
-
 	/* button mask */
 	rb_define_const(WRMT, "MASK_BUTTON_TWO",   INT2FIX(WRMT_MASK_BUTTON_TWO));
 	rb_define_const(WRMT, "MASK_BUTTON_ONE",   INT2FIX(WRMT_MASK_BUTTON_ONE));
@@ -491,13 +669,18 @@ void Init_wiiremote()
 	rb_define_const(WRMT, "IO_SUCCESS",          INT2FIX(WRMT_IO_SUCCESS));
 
 	/* class method */
-	rb_define_singleton_method(WRMT, "init", WRMT_init, 0);
-	rb_define_singleton_method(WRMT, "quit", WRMT_quit, 0);
-	rb_define_singleton_method(WRMT, "update", WRMT_update, 0);
-	rb_define_singleton_method(WRMT, "poll", WRMT_poll, 0);
-	rb_define_singleton_method(WRMT, "num_wiiremote", WRMT_num_wiiremote, 0);
-	rb_define_singleton_method(WRMT, "wiiremote_at", WRMT_wiiremote_at, 1);
-	rb_define_singleton_method(WRMT, "wiiremotes", WRMT_wiiremotes, 0);
+	rb_define_module_function(WRMT, "init", WRMT_init, 0);
+	rb_define_module_function(WRMT, "quit", WRMT_quit, 0);
+	rb_define_module_function(WRMT, "update_all", WRMT_update_all, 0);
+	rb_define_module_function(WRMT, "poll", WRMT_poll, 0);
+	rb_define_module_function(WRMT, "num_wiiremote", WRMT_num_wiiremote, 0);
+	rb_define_module_function(WRMT, "wiiremote_at", WRMT_wiiremote_at, 1);
+	rb_define_module_function(WRMT, "wiiremotes", WRMT_wiiremotes, 0);
+
+	/* button check utility */
+	rb_define_module_function(WRMT, "check_button", WRMT_check_button_any, 2);
+	rb_define_module_function(WRMT, "check_button_any", WRMT_check_button_any, 2);
+	rb_define_module_function(WRMT, "check_button_all", WRMT_check_button_all, 2);
 
 	/**
 	* WRMT::WiiRemote
@@ -508,7 +691,9 @@ void Init_wiiremote()
 	rb_define_method(WiiRemote, "open", WiiRemote_open, 0);
 	rb_define_method(WiiRemote, "opened?", WiiRemote_opened_p, 0);
 	rb_define_method(WiiRemote, "close", WiiRemote_close, 0);
+	rb_define_method(WiiRemote, "update", WiiRemote_update, 0);
 
+	/* enable/disable function */
 	rb_define_method(WiiRemote, "enabled?", WiiRemote_enabled_p, 1);
 	rb_define_method(WiiRemote, "continuous", WiiRemote_continuous, 0);
 	rb_define_method(WiiRemote, "motion", WiiRemote_motion, 0);
@@ -520,12 +705,26 @@ void Init_wiiremote()
 	rb_define_method(WiiRemote, "ir=", WiiRemote_set_ir, 1);
 	rb_define_method(WiiRemote, "speaker=", WiiRemote_set_speaker, 1);
 
+	/* get/set device state */
 	rb_define_method(WiiRemote, "state", WiiRemote_state, 1);
-	rb_define_alias(WiiRemote, "[]", "state");
 	rb_define_method(WiiRemote, "set_state", WiiRemote_set_state, 2);
+	rb_define_alias(WiiRemote, "[]", "state");
 	rb_define_alias(WiiRemote, "[]=", "set_state");
 
-	/* utility methods */
+	/* getter utilities */
+	rb_define_method(WiiRemote, "buttons", WiiRemote_buttons, 0);
+	rb_define_method(WiiRemote, "motion_x", WiiRemote_motion_x, 0);
+	rb_define_method(WiiRemote, "motion_y", WiiRemote_motion_y, 0);
+	rb_define_method(WiiRemote, "motion_z", WiiRemote_motion_z, 0);
+	rb_define_method(WiiRemote, "ir_found", WiiRemote_ir_found, 0);
+	rb_define_method(WiiRemote, "ir_size", WiiRemote_ir_size, 0);
+	rb_define_method(WiiRemote, "ir_x", WiiRemote_ir_x, 0);
+	rb_define_method(WiiRemote, "ir_y", WiiRemote_ir_y, 0);
+
+	/* speaker */
+	rb_define_method(WiiRemote, "play_sound", WiiRemote_play_sound, 3);
+
+	/* button check utility */
 	rb_define_method(WiiRemote, "check_button_any", WiiRemote_check_button_any, 1);
 	rb_define_method(WiiRemote, "check_button_all", WiiRemote_check_button_all, 1);
 	rb_define_alias(WiiRemote, "check_button", "check_button_any");
